@@ -6,11 +6,13 @@
 from IPython.core.interactiveshell import InteractiveShell
 from IPython.display import display
 from IPython.utils.capture import capture_output
+import inspect
+import pytest
 
+from .. import widget
 from ..widget import Widget
 from ..widget_button import Button
 
-
 def test_no_widget_view():
     # ensure IPython shell is instantiated
     # otherwise display() just calls print
@@ -49,9 +51,26 @@ def test_close_all():
     # create a couple of widgets
     widgets = [Button() for i in range(10)]
 
-    assert len(Widget._active_widgets) > 0, "expect active widgets"
-
+    assert len(widget._instances) > 0, "expect active widgets"
+    assert widget._instances[widgets[0].model_id] is widgets[0]
     # close all the widgets
     Widget.close_all()
 
-    assert len(Widget._active_widgets) == 0, "active widgets should be cleared"
+    assert len(widget._instances) == 0, "active widgets should be cleared"
+
+
+def test_compatibility():
+    button = Button()
+    assert widget._instances[button.model_id] is button
+    with pytest.deprecated_call() as record:
+        assert widget._instances is widget.Widget.widgets
+        assert widget._instances is widget.Widget._active_widgets
+        assert widget._registry is widget.Widget.widget_types
+        assert widget._registry is widget.Widget._widget_types
+
+        Widget.close_all()
+        assert not widget.Widget.widgets
+        assert not widget.Widget._active_widgets
+    caller_path = inspect.stack(context=0)[1].filename
+    assert all(x.filename == caller_path for x in record)
+    assert len(record) == 6
